from collections.abc import Iterable, Mapping
from contextlib import AbstractContextManager
from typing import Any, TypeVar, overload
from typing_extensions import Literal

from .client import Pipeline, Redis, _StrType

_T = TypeVar("_T")

HIREDIS_AVAILABLE: bool
CRYPTOGRAPHY_AVAILABLE: bool

@overload
def from_url(url: str, *, db: int = ..., decode_responses: Literal[True], **kwargs: Any) -> Redis[str]: ...
@overload
def from_url(url: str, *, db: int = ..., decode_responses: Literal[False] = ..., **kwargs: Any) -> Redis[bytes]: ...
def pipeline(redis_obj: Redis[_StrType]) -> AbstractContextManager[Pipeline[_StrType]]: ...
def str_if_bytes(value: str | bytes) -> str: ...
def safe_str(value: object) -> str: ...
def dict_merge(*dicts: Mapping[str, _T]) -> dict[str, _T]: ...
def list_keys_to_dict(key_list, callback): ...  # unused, alias for `dict.fromkeys`
def merge_result(command: object, res: Mapping[Any, Iterable[_T]]) -> list[_T]: ...



import asyncio
import enum
import ssl
from _typeshed import Incomplete
from collections.abc import Callable, Iterable, Mapping
from typing import Any, Protocol, overload
from typing_extensions import Literal, TypeAlias, TypedDict

from redis import RedisError
from redis.asyncio.retry import Retry
from redis.credentials import CredentialProvider
from redis.exceptions import ResponseError
from redis.typing import EncodableT, EncodedT

hiredis: Any
SYM_STAR: bytes
SYM_DOLLAR: bytes
SYM_CRLF: bytes
SYM_LF: bytes
SYM_EMPTY: bytes
SERVER_CLOSED_CONNECTION_ERROR: str

class _Sentinel(enum.Enum):
    sentinel: Any

SENTINEL: Any
MODULE_LOAD_ERROR: str
NO_SUCH_MODULE_ERROR: str
MODULE_UNLOAD_NOT_POSSIBLE_ERROR: str
MODULE_EXPORTS_DATA_TYPES_ERROR: str

class Encoder:
    encoding: Any
    encoding_errors: Any
    decode_responses: Any
    def __init__(self, encoding: str, encoding_errors: str, decode_responses: bool) -> None: ...
    def encode(self, value: EncodableT) -> EncodedT: ...
    def decode(self, value: EncodableT, force: bool = ...) -> EncodableT: ...

ExceptionMappingT: TypeAlias = Mapping[str, type[Exception] | Mapping[str, type[Exception]]]

class BaseParser:
    EXCEPTION_CLASSES: ExceptionMappingT
    def __init__(self, socket_read_size: int) -> None: ...
    def __del__(self) -> None: ...
    def parse_error(self, response: str) -> ResponseError: ...
    def on_disconnect(self) -> None: ...
    def on_connect(self, connection: Connection): ...
    async def read_response(self, disable_decoding: bool = ...) -> EncodableT | ResponseError | list[EncodableT] | None: ...

class PythonParser(BaseParser):
    encoder: Any
    def __init__(self, socket_read_size: int) -> None: ...
    def on_connect(self, connection: Connection): ...
    def on_disconnect(self) -> None: ...
    async def read_response(self, disable_decoding: bool = ...) -> EncodableT | ResponseError | None: ...

class HiredisParser(BaseParser):
    def __init__(self, socket_read_size: int) -> None: ...
    def on_connect(self, connection: Connection): ...
    def on_disconnect(self) -> None: ...
    async def read_from_socket(self) -> Literal[True]: ...
    async def read_response(self, disable_decoding: bool = ...) -> EncodableT | list[EncodableT]: ...

DefaultParser: type[PythonParser | HiredisParser]

class ConnectCallbackProtocol(Protocol):
    def __call__(self, connection: Connection): ...

class AsyncConnectCallbackProtocol(Protocol):
    async def __call__(self, connection: Connection): ...

ConnectCallbackT: TypeAlias = ConnectCallbackProtocol | AsyncConnectCallbackProtocol

class Connection:
    pid: Any
    host: Any
    port: Any
    db: Any
    username: Any
    client_name: Any
    password: Any
    socket_timeout: Any
    socket_connect_timeout: Any
    socket_keepalive: Any
    socket_keepalive_options: Any
    socket_type: Any
    retry_on_timeout: Any
    retry_on_error: list[type[RedisError]]
    retry: Retry
    health_check_interval: Any
    next_health_check: int
    ssl_context: Any
    encoder: Any
    redis_connect_func: ConnectCallbackT | None
    def __init__(
        self,
        *,
        host: str = ...,
        port: str | int = ...,
        db: str | int = ...,
        password: str | None = ...,
        socket_timeout: float | None = ...,
        socket_connect_timeout: float | None = ...,
        socket_keepalive: bool = ...,
        socket_keepalive_options: Mapping[int, int | bytes] | None = ...,
        socket_type: int = ...,
        retry_on_timeout: bool = ...,
        retry_on_error: list[type[RedisError]] | _Sentinel = ...,
        encoding: str = ...,
        encoding_errors: str = ...,
        decode_responses: bool = ...,
        parser_class: type[BaseParser] = ...,
        socket_read_size: int = ...,
        health_check_interval: float = ...,
        client_name: str | None = ...,
        username: str | None = ...,
        retry: Retry | None = ...,
        redis_connect_func: ConnectCallbackT | None = ...,
        encoder_class: type[Encoder] = ...,
        credential_provider: CredentialProvider | None = ...,
    ) -> None: ...
    def repr_pieces(self): ...
    def __del__(self) -> None: ...
    @property
    def is_connected(self): ...
    def register_connect_callback(self, callback) -> None: ...
    def clear_connect_callbacks(self) -> None: ...
    def set_parser(self, parser_class) -> None: ...
    async def connect(self) -> None: ...
    async def on_connect(self) -> None: ...
    async def disconnect(self, nowait: bool = ...) -> None: ...
    async def check_health(self) -> None: ...
    async def send_packed_command(self, command: bytes | str | Iterable[bytes], check_health: bool = ...): ...
    async def send_command(self, *args, **kwargs) -> None: ...
    @overload
    async def read_response(self, *, timeout: float) -> Incomplete | None: ...
    @overload
    async def read_response(self, disable_decoding: bool, timeout: float) -> Incomplete | None: ...
    @overload
    async def read_response(self, disable_decoding: bool = False, timeout: None = None): ...
    def pack_command(self, *args: EncodableT) -> list[bytes]: ...
    def pack_commands(self, commands: Iterable[Iterable[EncodableT]]) -> list[bytes]: ...

class SSLConnection(Connection):
    ssl_context: Any
    def __init__(
        self,
        ssl_keyfile: str | None = ...,
        ssl_certfile: str | None = ...,
        ssl_cert_reqs: str = ...,
        ssl_ca_certs: str | None = ...,
        ssl_ca_data: str | None = ...,
        ssl_check_hostname: bool = ...,
        **kwargs,
    ) -> None: ...
    @property
    def keyfile(self): ...
    @property
    def certfile(self): ...
    @property
    def cert_reqs(self): ...
    @property
    def ca_certs(self): ...
    @property
    def ca_data(self): ...
    @property
    def check_hostname(self): ...

class RedisSSLContext:
    keyfile: Any
    certfile: Any
    cert_reqs: Any
    ca_certs: Any
    ca_data: Any
    check_hostname: Any
    context: Any
    def __init__(
        self,
        keyfile: str | None = ...,
        certfile: str | None = ...,
        cert_reqs: str | None = ...,
        ca_certs: str | None = ...,
        ca_data: str | None = ...,
        check_hostname: bool = ...,
    ) -> None: ...
    def get(self) -> ssl.SSLContext: ...

class UnixDomainSocketConnection(Connection):
    pid: Any
    path: Any
    db: Any
    username: Any
    client_name: Any
    password: Any
    socket_timeout: Any
    socket_connect_timeout: Any
    retry_on_timeout: Any
    retry_on_error: list[type[RedisError]]
    retry: Any
    health_check_interval: Any
    next_health_check: int
    redis_connect_func: ConnectCallbackT | None
    encoder: Any
    def __init__(
        self,
        *,
        path: str = ...,
        db: str | int = ...,
        username: str | None = ...,
        password: str | None = ...,
        socket_timeout: float | None = ...,
        socket_connect_timeout: float | None = ...,
        encoding: str = ...,
        encoding_errors: str = ...,
        decode_responses: bool = ...,
        retry_on_timeout: bool = ...,
        retry_on_error: list[type[RedisError]] | _Sentinel = ...,
        parser_class: type[BaseParser] = ...,
        socket_read_size: int = ...,
        health_check_interval: float = ...,
        client_name: str | None = ...,
        retry: Retry | None = ...,
        redis_connect_func: ConnectCallbackT | None = ...,
        credential_provider: CredentialProvider | None = ...,
    ) -> None: ...
    def repr_pieces(self) -> Iterable[tuple[str, str | int]]: ...

FALSE_STRINGS: Any

def to_bool(value) -> bool | None: ...

URL_QUERY_ARGUMENT_PARSERS: Mapping[str, Callable[..., object]]

class ConnectKwargs(TypedDict):
    username: str
    password: str
    connection_class: type[Connection]
    host: str
    port: int
    db: int
    path: str

def parse_url(url: str) -> ConnectKwargs: ...

class ConnectionPool:
    @classmethod
    def from_url(cls, url: str, **kwargs) -> ConnectionPool: ...
    connection_class: Any
    connection_kwargs: Any
    max_connections: Any
    encoder_class: Any
    def __init__(
        self, connection_class: type[Connection] = ..., max_connections: int | None = ..., **connection_kwargs
    ) -> None: ...
    pid: Any
    def reset(self) -> None: ...
    async def get_connection(self, command_name, *keys, **options): ...
    def get_encoder(self): ...
    def make_connection(self): ...
    async def release(self, connection: Connection): ...
    def owns_connection(self, connection: Connection): ...
    async def disconnect(self, inuse_connections: bool = ...): ...

class BlockingConnectionPool(ConnectionPool):
    queue_class: Any
    timeout: Any
    def __init__(
        self,
        max_connections: int = ...,
        timeout: int | None = ...,
        connection_class: type[Connection] = ...,
        queue_class: type[asyncio.Queue[Any]] = ...,
        **connection_kwargs,
    ) -> None: ...
    pool: Any
    pid: Any
    def reset(self) -> None: ...
    def make_connection(self): ...
    async def get_connection(self, command_name, *keys, **options): ...
    async def release(self, connection: Connection): ...
    async def disconnect(self, inuse_connections: bool = ...): ...



from _typeshed import Incomplete
from typing import Any

# TODO: define and use:
# from redis.asyncio.cluster import ClusterNode

class CommandsParser:
    async def initialize(self, node: Incomplete | None = ...) -> None: ...  # TODO: ClusterNode
    async def get_keys(self, *args: Any) -> tuple[str, ...] | None: ...



from _typeshed import Incomplete
from types import TracebackType
from typing import Any, ClassVar, Protocol
from typing_extensions import Self

from redis.client import Redis

class _Local(Protocol):
    token: str | bytes | None

class Lock:
    LUA_EXTEND_SCRIPT: ClassVar[str]
    LUA_REACQUIRE_SCRIPT: ClassVar[str]
    LUA_RELEASE_SCRIPT: ClassVar[str]
    lua_extend: ClassVar[Incomplete | None]
    lua_reacquire: ClassVar[Incomplete | None]
    lua_release: ClassVar[Incomplete | None]
    local: _Local
    def __init__(
        self,
        redis: Redis[Any],
        name: str,
        timeout: float | None = ...,
        sleep: float = ...,
        blocking: bool = ...,
        blocking_timeout: float | None = ...,
        thread_local: bool = ...,
    ) -> None: ...
    def register_scripts(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> bool | None: ...
    def acquire(
        self,
        sleep: float | None = ...,
        blocking: bool | None = ...,
        blocking_timeout: float | None = ...,
        token: str | bytes | None = ...,
    ) -> bool: ...
    def do_acquire(self, token: str | bytes) -> bool: ...
    def locked(self) -> bool: ...
    def owned(self) -> bool: ...
    def release(self) -> None: ...
    def do_release(self, expected_token: str | bytes) -> None: ...
    def extend(self, additional_time: float, replace_ttl: bool = ...) -> bool: ...
    def do_extend(self, additional_time: float, replace_ttl: bool) -> bool: ...
    def reacquire(self) -> bool: ...
    def do_reacquire(self) -> bool: ...



from redis.asyncio.client import Redis as Redis, StrictRedis as StrictRedis
from redis.asyncio.connection import (
    BlockingConnectionPool as BlockingConnectionPool,
    Connection as Connection,
    ConnectionPool as ConnectionPool,
    SSLConnection as SSLConnection,
    UnixDomainSocketConnection as UnixDomainSocketConnection,
)
from redis.asyncio.sentinel import (
    Sentinel as Sentinel,
    SentinelConnectionPool as SentinelConnectionPool,
    SentinelManagedConnection as SentinelManagedConnection,
    SentinelManagedSSLConnection as SentinelManagedSSLConnection,
)
from redis.asyncio.utils import from_url as from_url
from redis.exceptions import (
    AuthenticationError as AuthenticationError,
    AuthenticationWrongNumberOfArgsError as AuthenticationWrongNumberOfArgsError,
    BusyLoadingError as BusyLoadingError,
    ChildDeadlockedError as ChildDeadlockedError,
    ConnectionError as ConnectionError,
    DataError as DataError,
    InvalidResponse as InvalidResponse,
    PubSubError as PubSubError,
    ReadOnlyError as ReadOnlyError,
    RedisError as RedisError,
    ResponseError as ResponseError,
    TimeoutError as TimeoutError,
    WatchError as WatchError,
)

__all__ = [
    "AuthenticationError",
    "AuthenticationWrongNumberOfArgsError",
    "BlockingConnectionPool",
    "BusyLoadingError",
    "ChildDeadlockedError",
    "Connection",
    "ConnectionError",
    "ConnectionPool",
    "DataError",
    "from_url",
    "InvalidResponse",
    "PubSubError",
    "ReadOnlyError",
    "Redis",
    "RedisError",
    "ResponseError",
    "Sentinel",
    "SentinelConnectionPool",
    "SentinelManagedConnection",
    "SentinelManagedSSLConnection",
    "SSLConnection",
    "StrictRedis",
    "TimeoutError",
    "UnixDomainSocketConnection",
    "WatchError",
]



from _typeshed import Incomplete
from typing import Any

class TSInfo:
    rules: list[Any]
    labels: list[Any]
    sourceKey: Incomplete | None
    chunk_count: Incomplete | None
    memory_usage: Incomplete | None
    total_samples: Incomplete | None
    retention_msecs: Incomplete | None
    last_time_stamp: Incomplete | None
    first_time_stamp: Incomplete | None

    max_samples_per_chunk: Incomplete | None
    chunk_size: Incomplete | None
    duplicate_policy: Incomplete | None
    def __init__(self, args) -> None: ...



from abc import ABC, abstractmethod

class AbstractBackoff(ABC):
    def reset(self) -> None: ...
    @abstractmethod
    def compute(self, failures: int) -> float: ...

class ConstantBackoff(AbstractBackoff):
    def __init__(self, backoff: int) -> None: ...
    def compute(self, failures: int) -> float: ...

class NoBackoff(ConstantBackoff):
    def __init__(self) -> None: ...

class ExponentialBackoff(AbstractBackoff):
    def __init__(self, cap: float = ..., base: float = ...) -> None: ...
    def compute(self, failures: int) -> float: ...

class FullJitterBackoff(AbstractBackoff):
    def __init__(self, cap: float = ..., base: float = ...) -> None: ...
    def compute(self, failures: int) -> float: ...

class EqualJitterBackoff(AbstractBackoff):
    def __init__(self, cap: float = ..., base: float = ...) -> None: ...
    def compute(self, failures: int) -> float: ...

class DecorrelatedJitterBackoff(AbstractBackoff):
    def __init__(self, cap: float = ..., base: float = ...) -> None: ...
    def compute(self, failures: int) -> float: ...



from collections.abc import Awaitable, Callable, Iterable
from typing import TypeVar

from redis.backoff import AbstractBackoff
from redis.exceptions import RedisError

_T = TypeVar("_T")

class Retry:
    def __init__(self, backoff: AbstractBackoff, retries: int, supported_errors: tuple[type[RedisError], ...] = ...) -> None: ...
    def update_supported_errors(self, specified_errors: Iterable[type[RedisError]]) -> None: ...
    async def call_with_retry(self, do: Callable[[], Awaitable[_T]], fail: Callable[[RedisError], Awaitable[object]]) -> _T: ...



from _typeshed import Incomplete

from .commands import SearchCommands

class Search(SearchCommands):
    class BatchIndexer:
        def __init__(self, client, chunk_size: int = ...) -> None: ...
        def add_document(
            self,
            doc_id,
            nosave: bool = ...,
            score: float = ...,
            payload: Incomplete | None = ...,
            replace: bool = ...,
            partial: bool = ...,
            no_create: bool = ...,
            **fields,
        ): ...
        def add_document_hash(self, doc_id, score: float = ..., replace: bool = ...): ...
        def commit(self): ...

    def __init__(self, client, index_name: str = ...) -> None: ...



from _typeshed import Incomplete
from collections.abc import Mapping
from typing import Any
from typing_extensions import Literal, TypeAlias

from .aggregation import AggregateRequest, AggregateResult, Cursor
from .query import Query
from .result import Result

_QueryParams: TypeAlias = Mapping[str, str | float]

NUMERIC: Literal["NUMERIC"]

CREATE_CMD: Literal["FT.CREATE"]
ALTER_CMD: Literal["FT.ALTER"]
SEARCH_CMD: Literal["FT.SEARCH"]
ADD_CMD: Literal["FT.ADD"]
ADDHASH_CMD: Literal["FT.ADDHASH"]
DROP_CMD: Literal["FT.DROP"]
EXPLAIN_CMD: Literal["FT.EXPLAIN"]
EXPLAINCLI_CMD: Literal["FT.EXPLAINCLI"]
DEL_CMD: Literal["FT.DEL"]
AGGREGATE_CMD: Literal["FT.AGGREGATE"]
PROFILE_CMD: Literal["FT.PROFILE"]
CURSOR_CMD: Literal["FT.CURSOR"]
SPELLCHECK_CMD: Literal["FT.SPELLCHECK"]
DICT_ADD_CMD: Literal["FT.DICTADD"]
DICT_DEL_CMD: Literal["FT.DICTDEL"]
DICT_DUMP_CMD: Literal["FT.DICTDUMP"]
GET_CMD: Literal["FT.GET"]
MGET_CMD: Literal["FT.MGET"]
CONFIG_CMD: Literal["FT.CONFIG"]
TAGVALS_CMD: Literal["FT.TAGVALS"]
ALIAS_ADD_CMD: Literal["FT.ALIASADD"]
ALIAS_UPDATE_CMD: Literal["FT.ALIASUPDATE"]
ALIAS_DEL_CMD: Literal["FT.ALIASDEL"]
INFO_CMD: Literal["FT.INFO"]
SUGADD_COMMAND: Literal["FT.SUGADD"]
SUGDEL_COMMAND: Literal["FT.SUGDEL"]
SUGLEN_COMMAND: Literal["FT.SUGLEN"]
SUGGET_COMMAND: Literal["FT.SUGGET"]
SYNUPDATE_CMD: Literal["FT.SYNUPDATE"]
SYNDUMP_CMD: Literal["FT.SYNDUMP"]

NOOFFSETS: Literal["NOOFFSETS"]
NOFIELDS: Literal["NOFIELDS"]
STOPWORDS: Literal["STOPWORDS"]
WITHSCORES: Literal["WITHSCORES"]
FUZZY: Literal["FUZZY"]
WITHPAYLOADS: Literal["WITHPAYLOADS"]

class SearchCommands:
    def batch_indexer(self, chunk_size: int = ...): ...
    def create_index(
        self,
        fields,
        no_term_offsets: bool = ...,
        no_field_flags: bool = ...,
        stopwords: Incomplete | None = ...,
        definition: Incomplete | None = ...,
        max_text_fields: bool = ...,  # added in 4.1.1
        temporary: Incomplete | None = ...,  # added in 4.1.1
        no_highlight: bool = ...,  # added in 4.1.1
        no_term_frequencies: bool = ...,  # added in 4.1.1
        skip_initial_scan: bool = ...,  # added in 4.1.1
    ): ...
    def alter_schema_add(self, fields): ...
    def dropindex(self, delete_documents: bool = ...): ...
    def add_document(
        self,
        doc_id,
        nosave: bool = ...,
        score: float = ...,
        payload: Incomplete | None = ...,
        replace: bool = ...,
        partial: bool = ...,
        language: Incomplete | None = ...,
        no_create: bool = ...,
        **fields,
    ): ...
    def add_document_hash(self, doc_id, score: float = ..., language: Incomplete | None = ..., replace: bool = ...): ...
    def delete_document(self, doc_id, conn: Incomplete | None = ..., delete_actual_document: bool = ...): ...
    def load_document(self, id): ...
    def get(self, *ids): ...
    def info(self): ...
    def get_params_args(self, query_params: _QueryParams) -> list[Any]: ...
    def search(self, query: str | Query, query_params: _QueryParams | None = ...) -> Result: ...
    def explain(self, query: str | Query, query_params: _QueryParams | None = ...): ...
    def explain_cli(self, query): ...
    def aggregate(self, query: AggregateRequest | Cursor, query_params: _QueryParams | None = ...) -> AggregateResult: ...
    def profile(
        self, query: str | Query | AggregateRequest, limited: bool = ..., query_params: Mapping[str, str | float] | None = ...
    ) -> tuple[Incomplete, Incomplete]: ...
    def spellcheck(
        self, query, distance: Incomplete | None = ..., include: Incomplete | None = ..., exclude: Incomplete | None = ...
    ): ...
    def dict_add(self, name, *terms): ...
    def dict_del(self, name, *terms): ...
    def dict_dump(self, name): ...
    def config_set(self, option: str, value: str) -> bool: ...
    def config_get(self, option: str) -> dict[str, str]: ...
    def tagvals(self, tagfield): ...
    def aliasadd(self, alias): ...
    def aliasupdate(self, alias): ...
    def aliasdel(self, alias): ...
    def sugadd(self, key, *suggestions, **kwargs): ...
    def suglen(self, key): ...
    def sugdel(self, key, string): ...
    def sugget(self, key, prefix, fuzzy: bool = ..., num: int = ..., with_scores: bool = ..., with_payloads: bool = ...): ...
    def synupdate(self, groupid, skipinitial: bool = ..., *terms): ...
    def syndump(self): ...



from typing import Any

from .commands import *
from .info import BFInfo as BFInfo, CFInfo as CFInfo, CMSInfo as CMSInfo, TDigestInfo as TDigestInfo, TopKInfo as TopKInfo

class AbstractBloom:
    @staticmethod
    def append_items(params, items) -> None: ...
    @staticmethod
    def append_error(params, error) -> None: ...
    @staticmethod
    def append_capacity(params, capacity) -> None: ...
    @staticmethod
    def append_expansion(params, expansion) -> None: ...
    @staticmethod
    def append_no_scale(params, noScale) -> None: ...
    @staticmethod
    def append_weights(params, weights) -> None: ...
    @staticmethod
    def append_no_create(params, noCreate) -> None: ...
    @staticmethod
    def append_items_and_increments(params, items, increments) -> None: ...
    @staticmethod
    def append_values_and_weights(params, items, weights) -> None: ...
    @staticmethod
    def append_max_iterations(params, max_iterations) -> None: ...
    @staticmethod
    def append_bucket_size(params, bucket_size) -> None: ...

class CMSBloom(CMSCommands, AbstractBloom):
    client: Any
    commandmixin: Any
    execute_command: Any
    def __init__(self, client, **kwargs) -> None: ...

class TOPKBloom(TOPKCommands, AbstractBloom):
    client: Any
    commandmixin: Any
    execute_command: Any
    def __init__(self, client, **kwargs) -> None: ...

class CFBloom(CFCommands, AbstractBloom):
    client: Any
    commandmixin: Any
    execute_command: Any
    def __init__(self, client, **kwargs) -> None: ...

class TDigestBloom(TDigestCommands, AbstractBloom):
    client: Any
    commandmixin: Any
    execute_command: Any
    def __init__(self, client, **kwargs) -> None: ...

class BFBloom(BFCommands, AbstractBloom):
    client: Any
    commandmixin: Any
    execute_command: Any
    def __init__(self, client, **kwargs) -> None: ...



from collections.abc import Iterable
from datetime import datetime, timedelta
from typing import Protocol, TypeVar
from typing_extensions import TypeAlias

from redis.asyncio.connection import ConnectionPool as AsyncConnectionPool
from redis.connection import ConnectionPool

# The following type aliases exist at runtime.
EncodedT: TypeAlias = bytes | memoryview
DecodedT: TypeAlias = str | int | float
EncodableT: TypeAlias = EncodedT | DecodedT
AbsExpiryT: TypeAlias = int | datetime
ExpiryT: TypeAlias = float | timedelta
ZScoreBoundT: TypeAlias = float | str
BitfieldOffsetT: TypeAlias = int | str
_StringLikeT: TypeAlias = bytes | str | memoryview  # noqa: Y043
KeyT: TypeAlias = _StringLikeT
PatternT: TypeAlias = _StringLikeT
FieldT: TypeAlias = EncodableT
KeysT: TypeAlias = KeyT | Iterable[KeyT]
ChannelT: TypeAlias = _StringLikeT
GroupT: TypeAlias = _StringLikeT
ConsumerT: TypeAlias = _StringLikeT
StreamIdT: TypeAlias = int | _StringLikeT
ScriptTextT: TypeAlias = _StringLikeT
TimeoutSecT: TypeAlias = int | float | _StringLikeT
AnyKeyT = TypeVar("AnyKeyT", bytes, str, memoryview)  # noqa: Y001
AnyFieldT = TypeVar("AnyFieldT", bytes, str, memoryview)  # noqa: Y001
AnyChannelT = TypeVar("AnyChannelT", bytes, str, memoryview)  # noqa: Y001

class CommandsProtocol(Protocol):
    connection_pool: AsyncConnectionPool | ConnectionPool
    def execute_command(self, *args, **options): ...



from _typeshed import Incomplete

BF_RESERVE: str
BF_ADD: str
BF_MADD: str
BF_INSERT: str
BF_EXISTS: str
BF_MEXISTS: str
BF_SCANDUMP: str
BF_LOADCHUNK: str
BF_INFO: str
CF_RESERVE: str
CF_ADD: str
CF_ADDNX: str
CF_INSERT: str
CF_INSERTNX: str
CF_EXISTS: str
CF_DEL: str
CF_COUNT: str
CF_SCANDUMP: str
CF_LOADCHUNK: str
CF_INFO: str
CMS_INITBYDIM: str
CMS_INITBYPROB: str
CMS_INCRBY: str
CMS_QUERY: str
CMS_MERGE: str
CMS_INFO: str
TOPK_RESERVE: str
TOPK_ADD: str
TOPK_INCRBY: str
TOPK_QUERY: str
TOPK_COUNT: str
TOPK_LIST: str
TOPK_INFO: str
TDIGEST_CREATE: str
TDIGEST_RESET: str
TDIGEST_ADD: str
TDIGEST_MERGE: str
TDIGEST_CDF: str
TDIGEST_QUANTILE: str
TDIGEST_MIN: str
TDIGEST_MAX: str
TDIGEST_INFO: str

class BFCommands:
    def create(self, key, errorRate, capacity, expansion: Incomplete | None = ..., noScale: Incomplete | None = ...): ...
    def add(self, key, item): ...
    def madd(self, key, *items): ...
    def insert(
        self,
        key,
        items,
        capacity: Incomplete | None = ...,
        error: Incomplete | None = ...,
        noCreate: Incomplete | None = ...,
        expansion: Incomplete | None = ...,
        noScale: Incomplete | None = ...,
    ): ...
    def exists(self, key, item): ...
    def mexists(self, key, *items): ...
    def scandump(self, key, iter): ...
    def loadchunk(self, key, iter, data): ...
    def info(self, key): ...

class CFCommands:
    def create(
        self,
        key,
        capacity,
        expansion: Incomplete | None = ...,
        bucket_size: Incomplete | None = ...,
        max_iterations: Incomplete | None = ...,
    ): ...
    def add(self, key, item): ...
    def addnx(self, key, item): ...
    def insert(self, key, items, capacity: Incomplete | None = ..., nocreate: Incomplete | None = ...): ...
    def insertnx(self, key, items, capacity: Incomplete | None = ..., nocreate: Incomplete | None = ...): ...
    def exists(self, key, item): ...
    def delete(self, key, item): ...
    def count(self, key, item): ...
    def scandump(self, key, iter): ...
    def loadchunk(self, key, iter, data): ...
    def info(self, key): ...

class TOPKCommands:
    def reserve(self, key, k, width, depth, decay): ...
    def add(self, key, *items): ...
    def incrby(self, key, items, increments): ...
    def query(self, key, *items): ...
    def count(self, key, *items): ...
    def list(self, key, withcount: bool = ...): ...
    def info(self, key): ...

class TDigestCommands:
    def create(self, key, compression: int = ...): ...
    def reset(self, key): ...
    def add(self, key, values): ...
    def merge(self, destination_key, num_keys, *keys, compression: int | None = ..., override: bool = ...): ...
    def min(self, key): ...
    def max(self, key): ...
    def quantile(self, key, quantile, *quantiles): ...
    def cdf(self, key, value, *values): ...
    def info(self, key): ...

class CMSCommands:
    def initbydim(self, key, width, depth): ...
    def initbyprob(self, key, error, probability): ...
    def incrby(self, key, items, increments): ...
    def query(self, key, *items): ...
    def merge(self, destKey, numKeys, srcKeys, weights=...): ...
    def info(self, key): ...



from typing import Any

class Result:
    total: Any
    duration: Any
    docs: Any
    def __init__(self, res, hascontent, duration: int = ..., has_payload: bool = ..., with_scores: bool = ...) -> None: ...
